package com.tomwheeler.stocktrader.api;

import java.util.Collection;

import com.tomwheeler.stocktrader.api.events.StockAccountListener;

/**
 * <p>
 * This interface represents an account used to buy and sell stock.
 * </p> 
 * 
 * @author Tom Wheeler
 */
public interface StockAccount {

    /** @return the amount of money with which the account was created */
    public int getInitialDepositAmount();
    
    /** @return the amount of money available for buying stocks */
    public int getCashBalance();
    
    /** @return a Collection (of Stock instances) for all stocks held in this account */
    public Collection getStocks();
    
    /** @return the number of shares of the specified stock held in the account */
    public int getQuantity(Stock stock);
    
    /** @return the description of this account */
    public String getDescription();
    
    /** @param description the description of this account */
    public void setDescription(String description);
    
    /**
     * <p>
     * Purchases the specified stock. This debits the account by the number of
     * shares time the current market price, and adds the specified number of
     * shares to the account.
     * </p>
     * 
     * @param stock
     *           the stock to be purchased
     * @param qty
     *           the number of shares to buy
     * @return the confirmation message for the purchase
     * @throws IllegalArgumentException
     *           if the number of shares is equal to or less than zero, if
     *           there is insufficient funds in the account to cover the
     *           trade, or if the ticker symbol is invalid.
     */
    public String buy(String symbol, int qty) throws IllegalArgumentException;

    /**
     * <p>
     * Sells the specified stock, which must debit the account's holdings in the
     * number of shares sold of this stock, and credit the cash balance by the
     * current market price times the number of shares sold.
     * </p>
     * 
     * @param symbol
     *            the symbol of the stock to be sold
     * @param qty
     *           the number of shares to sell
     * @return the confirmation message for the sale
     * @throws IllegalArgumentException
     *           if the number of shares to sell is equal to or less than zero, 
     *           if there is insufficient holdings of this stock in the account
     *           account to cover the trade, or if the ticker symbol is invalid.
     */
    public String sell(String symbol, int qty) throws IllegalArgumentException;

    /** 
     * @return a collection (of Strings) that lists every transaction made
     * with this account.
     */
    public Collection getTransactionHistory();
    
    /** registers a listener with this account so that it can be notified of changes */
    public void addStockAccountListener(StockAccountListener listener);
    
}
