 package com.tomwheeler.stocktrader.engine;

import java.util.*;

import com.tomwheeler.stocktrader.api.*;
import com.tomwheeler.stocktrader.api.events.StockAccountChangeEvent;
import com.tomwheeler.stocktrader.api.events.StockAccountListener;

/**
 * <p>
 * This is a concrete implementation of the StockExchange interface, used for
 * the stocktrader game.
 * </p>
 *  
 * @author Tom Wheeler
 */
public class StockAccountImpl implements StockAccount {
    
    private int initialDeposit = 0;
    private int cashBalance = 0;
    private String description;
    private Game game;
    private Collection txHistory;
    private Vector acctListeners;
    private Map stockHoldings;  // key=stock, value=numShares

    public StockAccountImpl(Game game, int initialDeposit) {
        this(game, null, initialDeposit);
    }

    public StockAccountImpl(Game game, String description, int initialDeposit) {
        if (game == null) {
            throw new IllegalArgumentException("game argument cannot be null");
        }

        setDescription(description);
        this.game = game;
        this.initialDeposit = initialDeposit;
        this.cashBalance = initialDeposit;
        stockHoldings = new HashMap();
        txHistory = new ArrayList();
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.StockAccount#getInitialDepositAmount()
     */
    public int getInitialDepositAmount() {
        return initialDeposit;
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.StockAccount#getCashBalance()
     */
    public int getCashBalance() {
        return cashBalance;
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.StockAccount#getStocks()
     */
    public Collection getStocks() {
        return Collections.unmodifiableSet(stockHoldings.keySet()); 
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.StockAccount#getQuantity(com.tomwheeler.stocktrader.engine.api.Stock)
     */
    public int getQuantity(Stock stock) {
        Integer qty = (Integer)stockHoldings.get(stock);
        if (qty == null) {
            return 0;
        }

        return qty.intValue();
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.StockAccount#getDescription()
     */
    public String getDescription() {
        return description;
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.StockAccount#setDescription(java.lang.String)
     */
    public void setDescription(String description) {
        if (description == null || description.trim().length() == 0) {
            description = "Stock Account";
        }
        this.description = description;
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.StockAccount#buy(java.lang.String, int)
     */
    public String buy(String symbol, int qty) throws IllegalArgumentException {
        StockExchange exchange = game.getExchange();
        StockDataProvider dataProvider = game.getDataProvider();
        
        int currentDay = game.getCurrentDay();
        int currentPrice = game.getDataProvider().getPrice(symbol, currentDay);
        int totalPrice = qty * currentPrice;

        // no need for checking here; exchange throws IAE if cashBalance is too low
        String confMsg = exchange.buy(this, symbol, qty);
        txHistory.add(confMsg);
        
        cashBalance -= totalPrice;
        
        // add to holdings
        Stock stock = exchange.getStock(symbol);
        Integer nowHeld;
        if (stockHoldings.keySet().contains(stock)) {
            Integer currentlyHeld = (Integer)stockHoldings.get(stock);
            nowHeld = new Integer(currentlyHeld.intValue() + qty);
            stockHoldings.put(stock, nowHeld);
        } else {
            nowHeld = new Integer(qty);                        
        }
        stockHoldings.put(stock, nowHeld);
        
        StockAccountChangeEvent event = new StockAccountChangeEvent(confMsg,
                StockAccountChangeEvent.STOCKS_PURCHASED, symbol);        
        notifyAccountListeners(event);
        
        return confMsg;
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.StockAccount#sell(java.lang.String, int)
     */
    public String sell(String symbol, int qty) throws IllegalArgumentException {
        StockExchange exchange = game.getExchange();
        StockDataProvider dataProvider = game.getDataProvider();

        int currentDay = game.getCurrentDay();
        int currentPrice = game.getDataProvider().getPrice(symbol, currentDay);
        int totalPrice = qty * currentPrice;

        // no need for checking here; exchange throws IAE if cashBalance is too low
        String confMsg = exchange.sell(this, symbol, qty);
        txHistory.add(confMsg);
        
        cashBalance += totalPrice;
        
        // remove from holdings
        Stock stock = exchange.getStock(symbol);
        Integer currentlyHeld = (Integer)stockHoldings.get(stock);
        int nowHeld = currentlyHeld.intValue() - qty;
        if (nowHeld == 0) {
            stockHoldings.remove(stock);
        } else {
            stockHoldings.put(stock, new Integer(nowHeld));    
        }

        StockAccountChangeEvent event = new StockAccountChangeEvent(confMsg,
                StockAccountChangeEvent.STOCKS_SOLD, symbol);        
        notifyAccountListeners(event);
        
        return confMsg;
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.StockAccount#getTransactionHistory()
     */
    public Collection getTransactionHistory() {
        return Collections.unmodifiableCollection(txHistory);
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.StockAccount#addStockAccountListener(com.tomwheeler.stocktrader.engine.api.event.StockAccountListener)
     */
    public void addStockAccountListener(StockAccountListener listener) {
        if (acctListeners == null) {
            acctListeners = new Vector();
        }
        acctListeners.add(listener);        
    }
    
    /** convenience method to create a new event and notify registered listeners */
    private void notifyAccountListeners(StockAccountChangeEvent event) {
        if (acctListeners == null) {
            // no listeners registered
            return;
        }
        
        for (Iterator iter = acctListeners.iterator(); iter.hasNext();) {
            StockAccountListener listener = (StockAccountListener) iter.next();
            
            int type = event.getEventType();
            if (type == StockAccountChangeEvent.STOCKS_PURCHASED) {
                listener.stockPurchased(event);
            } else if (type == StockAccountChangeEvent.STOCKS_SOLD) {
                listener.stocksSold(event);
            } else {
                String msg = "unknown event type in StockDataProviderImpl.notifyListeners";
                throw new IllegalArgumentException(msg);                
            }
        }
    }
}
