package com.tomwheeler.stocktrader.engine;

import com.tomwheeler.stocktrader.api.*;

/**
 * <p>
 * This is a concrete implementation of the Stock interface, used for
 * the stocktrader game.
 * </p>
 *  
 * @author Tom Wheeler
 */
public final class StockImpl implements Stock {

    private Game game;
    private final String symbol;
    private final String name;
    private final Industry industry;
    
    /**
     * <p>
     * Creates a new instance of the specified stock and registers it 
     * with the exchange.
     * </p>
     * 
     * @param symbol the symbol for this stock
     * @param name the name of the corporation issuing this stock
     * @param industry the industry in which the issuer does business
     * 
     * @throws IllegalArgumentException if a stock with this symbol already exists, 
     * or if any of the arguments are null, or if the name or symbol arguments
     * are empty (or consist solely of whitespace).
     */
    public StockImpl(String symbol, String name, Industry industry) throws IllegalArgumentException {
        this.symbol = symbol;
        this.name = name;
        this.industry = industry;
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.Stock#getSymbol()
     */
    public String getSymbol() {
        return symbol;
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.Stock#getName()
     */
    public String getName() {
        return name;
    }

    /* (non-Javadoc)
     * @see com.tomwheeler.stocktrader.engine.api.Stock#getIndustry()
     */
    public Industry getIndustry() {
        return industry;
    }
    
    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return symbol.hashCode();
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        // symbol is unique within the exchange; no need to check other fields
        if (this == obj) {
            return true;
        }

        if (!(obj instanceof Stock)) {
            return false;
        }

        String thatSymbol = ((Stock)obj).getSymbol();
        return symbol.equals(thatSymbol);
    }    

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        StringBuffer buff = new StringBuffer();

        buff.append("StockImpl: ");
        buff.append("[Symbol=");
        buff.append(symbol);
        buff.append(",[Name=");
        buff.append(name);
        buff.append(",[Industry=");
        buff.append(industry);
        
        return buff.toString();
    }
}
